<?php

namespace Drupal\session_inspector;

use Drupal\Core\Database\Connection;
use Drupal\Core\Session\AccountInterface;

/**
 * Implements SessionInspectorInterface.
 *
 * Allows sessions to be inspected and deleted.
 *
 * @package Drupal\session_inspector
 */
class SessionInspector implements SessionInspectorInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs a SessionInspector object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public function getSessions(AccountInterface $account):array {
    /** @var \Drupal\Core\Database\Query\Select $query */
    $query = $this->database->select('sessions', 's');
    $query->fields('s', ['uid', 'sid', 'hostname', 'timestamp', 'session']);
    $query->condition('s.uid', $account->id());
    $query->orderBy('timestamp', 'DESC');

    $results = $query->execute()->fetchAll();

    foreach ($results as $i => $result) {
      $exploded = explode('|', $result->session);
      $session_data = unserialize($exploded[1]);
      $results[$i]->browser = $session_data['browser'];
      unset($results[$i]->session);
    }

    return $results;
  }

  /**
   * {@inheritdoc}
   */
  public function getSession($sid): array {
    /** @var \Drupal\Core\Database\Query\Select $query */
    $query = $this->database->select('sessions', 's');
    $query->fields('s', ['uid', 'sid', 'hostname', 'timestamp']);
    $query->condition('s.sid', $sid);

    return $query->execute()->fetch(\PDO::FETCH_ASSOC);
  }

  /**
   * {@inheritdoc}
   */
  public function destroySession(string $session_id):void {
    $query = $this->database->delete('sessions');
    $query->condition('sid', $session_id);
    $query->execute();
  }

}
